#include <kglobal.h>
#include <klocale.h>
#include <kapplication.h>
#include <kmessagebox.h>
#include "appler.h"
#include "abbr.h"

appler::appler(const QString& configFile, Type type, int actions, QWidget *parent, const char *name) 
    : KPanelApplet(configFile, type, actions, parent, name)
{
  codec = QTextCodec::codecForName("KOI8-R");
  
  setBackgroundMode(X11ParentRelative);
  
  input = new KHistoryCombo(this);
  input->setFocus();
  input->clearEdit();
  input->lineEdit()->installEventFilter( this );
  input->setDuplicatesEnabled( FALSE );
  QToolTip::add( input, i18n("You can use regular expression for searching") );
  
  connect(input, SIGNAL(activated(const QString&)), SLOT(showWord(const QString&)));
  
  QSettings s;
  QStringList list = s.readListEntry( "/appler/main/wordsList" );
  input->setHistoryItems( list );
  DictName = s.readEntry( "/appler/main/dictFile", "/usr/local/share/dict/Mueller24.koi" );
  useRegExp = s.readBoolEntry( "/appler/main/useRegExp", FALSE );
  loadMullerMap();
}

appler::~appler()
{
  QSettings s;
  s.writeEntry( "/appler/main/wordsList", input->historyItems() );
}

void appler::about()
{
  KMessageBox::information(0, i18n("Muller Dictionary KDE Applet v.0.1"), i18n("About") );
}


void appler::help()
{
  KMessageBox::informationList(0, i18n("<b>Abbreviations</b>"),
         QStringList::split( '\n',codec->toUnicode( abbrs ) ), i18n("Help") );
}

void appler::preferences()
{
  setup dlg;
  if ( dlg.exec()==QDialog::Accepted ) {
    QSettings s;
    DictName = s.readEntry( "/appler/main/dictFile" );
    useRegExp = s.readBoolEntry( "/appler/main/useRegExp", FALSE );
  }
}

int appler::widthForHeight(int height) const
{
  return 150;
}

int appler::heightForWidth(int width) const
{
  return height();
}

void appler::resizeEvent(QResizeEvent *e)
{
  input->reparent(this, QPoint(0,0), true);
  int inputVOffset = height() - input->sizeHint().height() - 2;
  input->setGeometry(0, inputVOffset, width(), input->sizeHint().height());
  updateGeometry();
}

bool appler::eventFilter( QObject *o, QEvent * e)
{
  if ( ( e->type() == QEvent::MouseButtonRelease ) ||  
       ( e->type() == QEvent::MouseButtonPress ) ) {
     emit requestFocus();
  }
  return KPanelApplet::eventFilter( o, e );
}

void appler::showWord(const QString& word)
{
  QString article("");
  
  input->addToHistory(word);
  input->clearEdit();
  QRegExp defRE;
  if ( useRegExp ) {
    defRE = QRegExp( word.lower() );
  } else {
    defRE = QRegExp( "^"+word.lower().stripWhiteSpace()+"\\w*" );
  }
  QStringList lst = mullerMap.grep( defRE );
  
  if ( !lst.empty() ) {
     
    wordList *w = new wordList( lst, this, word.latin1(), Qt::WDestructiveClose | WType_TopLevel );
    connect( w, SIGNAL( selected(const QString&) ), this, SLOT( slShowArticle(const QString&) ) );
    w->show();
//  wordWindow *w = new wordWindow( 0, word.latin1(), WStyle_NoBorder|WStyle_StaysOnTop|WX11BypassWM|Qt::WDestructiveClose );
  }
}

void appler::slShowArticle(const QString& word)
{
  QStringList lst = mullerMap.grep( QRegExp( "^"+word+"@\\d*") );
  if ( !lst.empty() ) {
    QFile dict;
    dict.setName( DictName );
    if ( dict.open( IO_ReadOnly ) ) {
      QString article("");
      dict.at( lst.first().right( lst.first().length() - lst.first().find(QRegExp("@"),0)-1 ).toInt() );
      dict.readLine( article, dict.size() );
      wordWindow *w = new wordWindow( codec->toUnicode( article ) , this, word.latin1(), Qt::WDestructiveClose | WType_TopLevel );
      w->setCaption( word );
      w->show(); 
      w->setActiveWindow();
    }
  }
}

void appler::loadMullerMap()
{
  mullerMap.clear();
  QFile index;
  index.setName( DictName+".index" );
  if ( index.open( IO_ReadOnly ) ) {
    QTextStream stream( &index );
//    QProgressDialog progress( "Loading Index...", 0, index.size(), this, "progress", TRUE );
    while ( !index.atEnd() ) {
      QString str;
      if ( index.readLine( str, 80/*index.size()*/ )>0 ) {
        mullerMap << str.stripWhiteSpace();
      } 
//      progress.setProgress( index.at() );
     }
//    progress.setProgress( index.size() );
    index.close();
  }
}

extern "C"
{
    KPanelApplet* init( QWidget *parent, const QString configFile)
    {
        KGlobal::locale()->insertCatalogue("appler");
        return new appler(configFile, KPanelApplet::Stretch,
                             KPanelApplet::About | KPanelApplet::Help | KPanelApplet::Preferences,
                             parent, "appler");
    }
}

//////////////////////////  wordWindow  ////////////////////////

wordWindow::wordWindow( QString txt, QWidget *parent, const char *name, WFlags f )
  : QMainWindow( parent, name, f )
{
  QSettings s;
  
  text = new QTextBrowser(this);
  
  QRegExp rex("(\\d+>|\\w+>|\\d+\\.|_[IVX]+)");
  int pos=txt.find( rex, 0 );
  while ( pos!=-1 ) {
    txt.insert( pos, "<p>" );
    pos=pos+8;
    pos=txt.find( rex, pos );
  }
  enWord = txt.left( txt.find(QRegExp( "  " ),0) );
  txt.replace( enWord, "<font color=darkblue><b>"+enWord+"</b></font>" );
  KURL mname;
  mname.setPath( s.readEntry( "/appler/main/dictFile", "/" ) );
  QString sAudio = s.readEntry( "/appler/main/dir", "/usr/local/share/erss/sounds" )+"/"+enWord+".wav";
  if ( QFile::exists( sAudio ) ) 
    txt.insert( txt.find(QRegExp( "  " ),0)+1,
                QString("<a href=\"#play\"><img src=\"%1/sound.png\"></a>").arg( mname.directory() ) );
  text->setText( txt );
  text->setReadOnly( TRUE );
  connect( text, SIGNAL( linkClicked( const QString & ) ),
            this, SLOT( slPlaySound( const QString & ) ) );
  setCentralWidget(text);

  resize( QSize( s.readNumEntry( "/appler/wordWindow/width", 200 ),
                 s.readNumEntry( "/appler/wordWindow/height", 100 ) ) );
                 
  if ( ( dispatcher = new KArtsDispatcher() ) ) {
     server = Arts::Reference("global:Arts_SoundServer");
  }
}

wordWindow::~wordWindow()
{
  QSettings s;
  s.writeEntry( "/appler/wordWindow/width", width() );
  s.writeEntry( "/appler/wordWindow/height", height() );
}

void wordWindow::keyPressEvent( QKeyEvent *e )
{
  switch( e->key() ) {
    case Qt::Key_Escape : {
      close();    
      break;
    }
    default: e->ignore();
  }
  QMainWindow::keyPressEvent( e );
}

void wordWindow::slPlaySound( const QString & )
{
  QSettings s;
  server.play(s.readEntry( "/appler/main/dir", "" )+"/"+enWord+".wav"); 
}

////////////////////// wordList ///////////////////////

wordList::wordList( QStringList lst, QWidget *parent, const char *name, WFlags f )
  : QMainWindow( parent, name, f )
{
  setCaption( name );
  lbWords = new maListBox(this);
  for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it ) {
    QString s = *it;
    lbWords->insertItem( s.left( s.find(QRegExp("@"),0) ) );
  };
  connect( lbWords, SIGNAL( doubleClicked( QListBoxItem * ) ),
            this, SLOT(lbDoubleClicked( QListBoxItem * ) ) );
  setCentralWidget(lbWords);
  QSettings s;
  resize( QSize( s.readNumEntry( "/appler/wordList/width", 100 ),
                 s.readNumEntry( "/appler/wordList/height", 200 ) ) );
}

wordList::~wordList()
{
  QSettings s;
  s.writeEntry( "/appler/wordList/width", width() );
  s.writeEntry( "/appler/wordList/height", height() );
}

void wordList::keyPressEvent( QKeyEvent *e )
{
  switch( e->key() ) {
    case Qt::Key_Escape : {
      close();    
      break;
    }
    case Qt::Key_Enter : case Qt::Key_Return : case Qt::Key_Space : {
      QListBoxItem *item = lbWords->item( lbWords->currentItem() );
      if ( item ) {
        emit selected( item->text() );
      }
      break;
    }
    default: e->ignore();
  }
  QMainWindow::keyPressEvent( e );
}

void wordList::lbDoubleClicked( QListBoxItem *item )
{
  if ( item ) emit selected( item->text() );
}

////////////////////////  setup dialog  ///////////////

setup::setup( QWidget *parent, const char *name )
  :  QDialog( parent, name )
{
  codec = QTextCodec::codecForName("KOI8-R");
  setCaption( i18n("Preferences") );
  
  QGridLayout *grid = new QGridLayout( this, 6, 2, 2, 2 );
  
  QLabel *label = new QLabel( this );
  label->setText( "Applet Setup" );
  label->setAlignment( Qt::AlignCenter );
  label->setFrameStyle( QFrame::Plain | QFrame::Panel );
  label->setPaletteBackgroundColor( QColor( 193, 255, 200 ) );
  grid->addMultiCellWidget( label, 0, 0, 0, 1 );
  
  leDict = new QLineEdit( this );
  grid->addWidget( leDict, 1, 0 );
  
  QPushButton *pb = new QPushButton( i18n("Select Dictionary"), this );
  connect( pb, SIGNAL( clicked() ), this, SLOT( slotSetDict() ) );
  grid->addWidget( pb, 1, 1 );
  
  leWords = new QLineEdit( this );
  grid->addWidget( leWords, 2, 0 );
  pb = new QPushButton( i18n("Sounds Directory"), this );
  connect( pb, SIGNAL( clicked() ), this, SLOT( slotSetDir() ) );
  grid->addWidget( pb, 2, 1 );
  
  QLabel *lb = new QLabel( this );
  lb->setText( i18n( "Make kind of index file for Muller Dictionary" ) );
  grid->addWidget( lb, 3, 0 );
  
  pb = new QPushButton( i18n("Make Index"), this );
  connect( pb, SIGNAL( clicked() ), this, SLOT( slotIndex() ) );
  grid->addWidget( pb, 3, 1 );
    
  cbRegExp = new QCheckBox( i18n("Use Regular Expression"), this );
  grid->addMultiCellWidget( cbRegExp, 4,4,0,1 );
  
  QHBox *hb = new QHBox( this );
  pb = new QPushButton( i18n("Ok"), hb );
  connect( pb, SIGNAL( clicked() ), this, SLOT( slotOkPressed() ) );
  pb->setDefault( TRUE );
  
  pb = new QPushButton( i18n("Cancel"), hb );
  connect( pb, SIGNAL( clicked() ), this, SLOT( slotCancelPressed() ) );
  grid->addMultiCellWidget( hb, 5,5,0,1 );
  
  resize( QSize(300, 100 ) );
  
  QSettings s;
  leDict->setText( s.readEntry( "/appler/main/dictFile", "" ) );
  leWords->setText( s.readEntry( "/appler/main/dir", "/" ) );
  cbRegExp->setChecked( s.readBoolEntry( "/appler/main/useRegExp", FALSE ) );
  
}

setup::~setup()
{
}

void setup::slotSetDict()
{
  QString fn = KFileDialog::getOpenFileName("/","*");
  if ( !fn.isEmpty() ) {
    leDict->setText( fn );
  }
}

void setup::slotSetDir()
{
  KURL url = KDirSelectDialog::selectDirectory( leWords->text(), TRUE );
  if ( !url.isEmpty() ) {
    leWords->setText( url.path() );
  }else{
    leWords->setText( "/" );
  }
}


void setup::slotOkPressed()
{
  QSettings s;
  s.writeEntry( "/appler/main/dictFile", leDict->text() );
  s.writeEntry( "/appler/main/dir", leWords->text() );
  s.writeEntry( "/appler/main/useRegExp", cbRegExp->isChecked() );
  accept();
}

void setup::slotCancelPressed()
{
  reject();
}

void setup::slotIndex()
{
  QStringList lst;
  QFile file;
  QFile index;
  file.setName( leDict->text() );
//  index.setName( leDict->text()+".index" );
  index.setName( leDict->text()+".lst" );
  QTextStream stream( &index );
  if ( file.open( IO_ReadOnly ) && index.open( IO_WriteOnly ) ) {
    QProgressDialog progress( "Indexing dictionary...", 0, file.size(), this, "progress", TRUE );
    while ( !file.atEnd() ) {
      QString str;
      if ( file.readLine( str, file.size() )>0 ) {
//        stream << codec->toUnicode( str.left( str.find(QRegExp("  "),0) ) ) <<"@"<< file.at()-str.length() << '\n';
         lst << codec->toUnicode( str.left( str.find(QRegExp("  "),0) ) )+"@"+file.at()-str.length();
      }
      progress.setProgress( file.at() );
     }
    progress.setProgress( file.size() );
    file.close();
    stream << &lst;
    index.close();
  }
}

////////////////////////   maListBox   ///////////////////////

maListBox::maListBox( QWidget *parent, const char *name, WFlags f )
  : QListBox( parent, name, f )
{
}

maListBox::~maListBox()
{
}

void maListBox::keyPressEvent( QKeyEvent *e )
{
  switch( e->key() ) {
    case Qt::Key_Enter : case Qt::Key_Return : case Qt::Key_Space : {
      e->ignore();
      break;
    }
    default: {
      e->accept();
      QListBox::keyPressEvent( e );
    }
  }
}

///////////////////////////    maDictionary     ////////////////////////

maDictionary::maDictionary( QString n )
{
  name = n;
  loadIndex();
}

maDictionary::~maDictionary()
{
}

void maDictionary::loadIndex()
{
  index.clear();
  QFile fi;
  fi.setName( name+".index" );
  if ( fi.open( IO_ReadOnly ) ) {
    QTextStream stream( &fi );
//    QProgressDialog progress( "Loading Index...", 0, index.size(), this, "progress", TRUE );
    while ( !fi.atEnd() ) {
      QString str;
      if ( fi.readLine( str, 80/*index.size()*/ )>0 ) {
        index << str.stripWhiteSpace();
      }
//      progress.setProgress( index.at() );
     }
//    progress.setProgress( index.size() );
    fi.close();
  }
}
