#ifndef APPLER_H
#define APPLER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kpanelapplet.h>
#include <kcombobox.h>
#include <qmainwindow.h>
#include <qtextbrowser.h> 
#include <qsettings.h>
#include <qpushbutton.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <kfiledialog.h>
#include <kdirselectdialog.h>
#include <qtextcodec.h>
#include <qlabel.h>
#include <qprogressdialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qstringlist.h>
#include <qlistbox.h>
#include <qtooltip.h>
#include <qcheckbox.h>
#include <arts/kplayobject.h>
#include <arts/kplayobjectfactory.h>
#include <arts/kartsdispatcher.h>
#include <arts/kartsserver.h>

typedef QMap<QString, int> simap;
class maListBox;

class maDictionary {
public:
  maDictionary( QString n );
  ~maDictionary();
private:
  void loadIndex();
  QString name;
  QStringList index;
};


class appler : public KPanelApplet
{
    Q_OBJECT
    
public:
    appler(const QString& configFile, Type t = Normal, int actions = 0,
        QWidget *parent = 0, const char *name = 0);
    ~appler();
    virtual int widthForHeight(int height) const;
    virtual int heightForWidth(int width) const;
    virtual void about();
    virtual void help();
    virtual void preferences();
    
protected:
    void resizeEvent(QResizeEvent *);
    bool eventFilter( QObject *, QEvent * );
protected slots:
    void showWord(const QString&);
    void slShowArticle(const QString&);
private:
    KHistoryCombo *input;
    QTextCodec *codec;
    QString DictName;
    QStringList mullerMap;
    void loadMullerMap();
    bool useRegExp;
};

class wordWindow : public QMainWindow
{
  Q_OBJECT
public:
  wordWindow( QString txt, QWidget *parent=0, const char *name=0, WFlags f=WType_TopLevel );
  ~wordWindow();
protected:
  void keyPressEvent( QKeyEvent *e );
protected slots:
  void slPlaySound( const QString & );
private:
  QTextBrowser *text;
  QString enWord;
  KArtsDispatcher* dispatcher;
  Arts::SoundServer server;
};

class wordList : public QMainWindow
{
  Q_OBJECT
public:
  wordList( QStringList lst, QWidget *parent=0, const char *name=0, WFlags f=WType_TopLevel );
  ~wordList();
protected:
  void keyPressEvent( QKeyEvent *e );
protected slots:
  void lbDoubleClicked( QListBoxItem *item );
signals:
  void selected(const QString&);
private:
  maListBox *lbWords;
};

class setup : public QDialog
{
  Q_OBJECT
public:
  setup( QWidget *parent=0, const char *name=0 );
  ~setup();
protected slots:
  void slotSetDict();
  void slotSetDir();
  void slotOkPressed();
  void slotCancelPressed();
  void slotIndex();
private:
  QLineEdit *leDict;
  QLineEdit *leWords;
  QTextCodec *codec;
  QCheckBox *cbRegExp;
};

class maListBox : public QListBox{
  Q_OBJECT
public:
  maListBox( QWidget *parent=0, const char *name=0, WFlags f=0 );
  ~maListBox();
protected:
  void keyPressEvent( QKeyEvent *e );
};

#endif
